import re
from sanatio.base_class import BaseValidator
from sanatio.utils.utils import regexs_dict


class EmailValidator(BaseValidator):
    """ 
    EmailValidator class is used to validate email address

    Methods
    -------

    isEmail(value: str, checkDNS: bool = False) -> bool:
        check if the string is email or not

    """

    def isEmail(self, value: str, checkDNS: bool = False) -> bool:
        """ check if the string is email or not """
        regex = regexs_dict.get('email_regex')
        return self.isvalidString(value) and bool(re.search(regex, value))



from sanatio.array_validator import ArrayValidator
from sanatio.date_validator import DateValidator
from sanatio.document_validator import DocumentValidator
from sanatio.email_validator import EmailValidator
from sanatio.number_validator import NumberValidator
from sanatio.other_validator import OtherValidator
from sanatio.password_validator import PasswordValidator
from sanatio.string_validator import StringValidator
from sanatio.username_validator import UsernameValidator
from sanatio.file_validator import FileValidator
from warnings import warn


class Sanatio(ArrayValidator,
              DateValidator,
              DocumentValidator,
              EmailValidator,
              NumberValidator,
              OtherValidator,
              PasswordValidator,
              StringValidator,
              UsernameValidator,
              FileValidator):
    """ Sanatio class for validating the data """
    def __init__(self):
        super().__init__()


class Validator(Sanatio):
    """ Validator class for validating the data """
    def __init__(self):
        warn("Validator class is deprecated, use Sanatio class instead", DeprecationWarning)



import re
import json

from sanatio.utils.utils import regexs_dict, country_json
from sanatio.base_class import BaseValidator
from sanatio.utils.checksum import EANCheckSum


class OtherValidator(BaseValidator):

    def isEAN13(self, value) -> bool:
        """ check if the string is EAN or not """
        value = str(value) if isinstance(value, int) else value
        return self.isLength(value, 13, 13) and EANCheckSum(value).verify()

    def isIMEI(self, value) -> bool:
        """ check if the string is IMEI or not """
        return len(value) == 15 or len(value) == 17

    def isIPV4(self, value: str) -> bool:
        """ check if the string is IP or not """
        regex = regexs_dict.get('ipv4_regex')
        return True if re.match(regex, value) else False

    def isIPV6(self, value: str) -> bool:
        regex = regexs_dict.get('ipv6_regex')
        return True if re.match(regex, value) else False

    def isSSN(self, value) -> bool:
        """ check if the string is SSN or not """
        regex = regexs_dict.get('ssn_regex')
        return True if re.match(regex, value) else False

    def isJSON(self, value) -> bool:
        """ check if the string is JSON or not """
        try:
            json.loads(value)
            return True
        except ValueError:
            return False

    def isJWT(self, value) -> bool:
        """ check if the string is JWT or not """
        regex = regexs_dict.get('jwt_regex')
        return True if re.match(regex, value) else False

    def isLatLong(self, value: str) -> bool:
        """ check if the string is lat long or not """
        regex = regexs_dict.get('lat_long_regex')
        return True if re.match(regex, value) else False

    def isMACAddress(self, value: str) -> bool:
        """ check if the string is MAC address or not """
        regex = regexs_dict.get('mac_address_regex')
        if self.isvalidString(value) and re.search(regex, value, re.IGNORECASE):
            return True
        return False

    def isPort(self, value: int) -> bool:
        """ check if the string is port or not
        A port number is a 16-bit unsigned integer,
        so it has a minimum value of 0 and a maximum value of 65535.
        """
        return value.isdigit() and 0 <= int(value) <= 65535

    def isSlug(self, value: str) -> bool:
        """ check if the string is slug or not """
        regex = regexs_dict.get('slug_regex')
        return True if re.match(regex, value) else False

    def isPostalCode(self, value, locale: str) -> bool:
        """ check if the string is postal code or not """
        country_data = country_json[locale]

        PostalCode = country_data['PostalCode']

        PostalCodeFormat = PostalCode['Format']
        PostalCodeRegex = PostalCode['Regex']
        MinPostalCodeLength = PostalCode['MinLength']
        MaxPostalCodeLength = PostalCode['MaxLength']

        if re.match(PostalCodeRegex, value) and re.match(PostalCodeFormat, value) \
                and self.isLength(str(value), MinPostalCodeLength, MaxPostalCodeLength):
            return True

        return False

    def isMobilePhone(self, value, locale: str) -> bool:
        """ check if the string is mobile phone or not """
        country_data = country_json[locale]
        MobileNumberRegex = country_data['MobileNumberRegex']
        if re.match(MobileNumberRegex, value):
            return True

    def isUUID(self, value: str) -> bool:
        """ check if the string is UUID or not """
        regex = regexs_dict.get('uuid_regex')
        return True if re.match(regex, value) else False



import os

from sanatio.base_class import BaseValidator


class FileValidator(BaseValidator):

    def isFile(self, value) -> bool:
        """ check if the string is file or not """
        return os.path.isfile(value)

    def isFileType(self, value, extensions) -> bool:
        """ check if the string is of a specific file type """
        return self.isFile(value) and value.endswith(extensions)

    def isImage(self, value) -> bool:
        """ check if the string is image or not """
        extensions = ('.jpg', '.jpeg', '.png', '.gif', '.bmp', '.svg', '.webp')
        return self.isFileType(value, extensions)

    def isVideo(self, value) -> bool:
        """ check if the string is video or not """
        extensions = ('.mp4', '.mkv', '.avi', '.flv', '.wmv', '.mov')
        return self.isFileType(value, extensions)

    def isAudio(self, value) -> bool:
        """ check if the string is audio or not """
        extensions = ('.mp3', '.wav', '.ogg', '.aac', '.wma', '.flac')
        return self.isFileType(value, extensions)

    def isPDF(self, value) -> bool:
        """ check if the string is PDF or not """
        extensions = ('.pdf')
        return self.isFileType(value, extensions)

    def isCSV(self, value) -> bool:
        """ check if the string is CSV or not """
        extensions = ('.csv')
        return self.isFileType(value, extensions)

    def isExcel(self, value) -> bool:
        """ check if the string is Excel or not """
        extensions = ('.xls', '.xlsx')
        return self.isFileType(value, extensions)

    def isWord(self, value) -> bool:
        """ check if the string is Word or not """
        extensions = ('.doc', '.docx')
        return self.isFileType(value, extensions)

    def isPowerPoint(self, value) -> bool:
        """ check if the string is PowerPoint or not """
        extensions = ('.ppt', '.pptx')
        return self.isFileType(value, extensions)

    def isText(self, value) -> bool:
        """ check if the string is Text or not """
        extensions = ('.txt')
        return self.isFileType(value, extensions)

    def isZip(self, value) -> bool:
        """ check if the string is Zip or not """
        extensions = ('.zip')
        return self.isFileType(value, extensions)

    def isGzip(self, value) -> bool:
        """ check if the string is Gzip or not """
        extensions = ('.gz')
        return self.isFileType(value, extensions)

    def isTar(self, value) -> bool:
        """ check if the string is Tar or not """
        extensions = ('.tar')
        return self.isFileType(value, extensions)

    def isRar(self, value) -> bool:
        """ check if the string is RAR or not """
        extensions = ('.rar')
        return self.isFileType(value, extensions)

    def is7z(self, value) -> bool:
        """ check if the string is 7z or not """
        extensions = ('.7z')
        return self.isFileType(value, extensions)

    def isJSON(self, value) -> bool:
        """ check if the string is JSON or not """
        extensions = ('.json')
        return self.isFileType(value, extensions)

    def isXML(self, value) -> bool:
        """ check if the string is XML or not """
        extensions = ('.xml')
        return self.isFileType(value, extensions)

    def isYAML(self, value) -> bool:
        """ check if the string is YAML or not """
        extensions = ('.yaml', '.yml')
        return self.isFileType(value, extensions)

    def isINI(self, value) -> bool:
        """ check if the string is INI or not """
        extensions = ('.ini')
        return self.isFileType(value, extensions)

    def isConfig(self, value) -> bool:
        """ check if the string is Config or not """
        extensions = ('.config')
        return self.isFileType(value, extensions)

    def isLog(self, value) -> bool:
        """ check if the string is Log or not """
        extensions = ('.log')
        return self.isFileType(value, extensions)

    def isSQL(self, value) -> bool:
        """ check if the string is SQL or not """
        extensions = ('.sql')
        return self.isFileType(value, extensions)

    def isHTML(self, value) -> bool:
        """ check if the string is HTML or not """
        extensions = ('.html')
        return self.isFileType(value, extensions)

    def isCSS(self, value) -> bool:
        """ check if the string is CSS or not """
        extensions = ('.css')
        return self.isFileType(value, extensions)

    def isJS(self, value) -> bool:
        """ check if the string is JS or not """
        extensions = ('.js')
        return self.isFileType(value, extensions)

    def isPHP(self, value) -> bool:
        """ check if the string is PHP or not """
        extensions = ('.php')
        return self.isFileType(value, extensions)

    def isPython(self, value) -> bool:
        """ check if the string is Python or not """
        extensions = ('.py')
        return self.isFileType(value, extensions)

    def isJava(self, value) -> bool:
        """ check if the string is Java or not """
        extensions = ('.java')
        return self.isFileType(value, extensions)

    def isC(self, value) -> bool:
        """ check if the string is C or not """
        extensions = ('.c')
        return self.isFileType(value, extensions)

    def isCPP(self, value) -> bool:
        """ check if the string is CPP or not """
        extensions = ('.cpp')
        return self.isFileType(value, extensions)

    def isCSharp(self, value) -> bool:
        """ check if the string is CSharp or not """
        extensions = ('.cs')
        return self.isFileType(value, extensions)

    def isRuby(self, value) -> bool:
        """ check if the string is Ruby or not """
        extensions = ('.rb')
        return self.isFileType(value, extensions)

    def isSwift(self, value) -> bool:
        """ check if the string is Swift or not """
        extensions = ('.swift')
        return self.isFileType(value, extensions)

    def isKotlin(self, value) -> bool:
        """ check if the string is Kotlin or not """
        extensions = ('.kt')
        return self.isFileType(value, extensions)

    def isR(self, value) -> bool:
        """ check if the string is R or not """
        extensions = ('.r')
        return self.isFileType(value, extensions)

    def isGo(self, value) -> bool:
        """ check if the string is Go or not """
        extensions = ('.go')
        return self.isFileType(value, extensions)

    def isScala(self, value) -> bool:
        """ check if the string is Scala or not """
        extensions = ('.scala')
        return self.isFileType(value, extensions)

    def isPerl(self, value) -> bool:
        """ check if the string is Perl or not """
        extensions = ('.pl')
        return self.isFileType(value, extensions)

    def isShell(self, value) -> bool:
        """ check if the string is Shell or not """
        extensions = ('.sh')
        return self.isFileType(value, extensions)

    def isBatch(self, value) -> bool:
        """ check if the string is Batch or not """
        extensions = ('.bat')
        return self.isFileType(value, extensions)

    def isPowerShell(self, value) -> bool:
        """ check if the string is PowerShell or not """
        extensions = ('.ps1')
        return self.isFileType(value, extensions)



import os
import json


class BaseValidator:
    """Base validator class for validating the data"""

    def isvalidString(self, value: str) -> bool:
        """Check if the string is valid or not"""
        return isinstance(value, str) and value != ''

    def isvalidNumber(self, value: int) -> bool:
        """Check if the number is valid or not"""
        return isinstance(value, (int, float))

    def isvalidBoolean(self, value) -> bool:
        """Check if the value is boolean or not"""
        return isinstance(value, bool)

    def removeSpaces(self, value) -> str:
        """Remove spaces from string"""
        return value.replace(" ", "") if self.isvalidString(value) else None
    
    def isFileExists(self, file_path: str) -> bool:
        """Check if the file exists or not"""
        return os.path.exists(file_path)
    
    def read_file(self, 
                  file_path: str, 
                  split_lines: bool = False, 
                  unique: bool = False, 
                  is_json: bool = False,
                  sep: str = '\n',
                  mode: str = 'r') -> list:
        """Read the file content"""
        if self.isFileExists(file_path):
            if is_json:
                return self.read_json(file_path)
            
            with open(file_path, mode) as file:
                data = file.read()
                if split_lines and unique:
                    return list(set(data.split(sep)))
                return data
        return None
        
    def read_json(self, file_path: str) -> dict:
        """Read the json file content"""
        if self.isFileExists(file_path):
            with open(file_path, 'r') as file:
                data = json.load(file)
                return data
        return {}

    def write_file(self, 
                   file_path: str, 
                   data: str, 
                   mode: str = 'w', 
                   sep: str = '') -> bool:
        """Write the data to the file"""
        with open(file_path, mode) as file:
            for line in data:
                file.write(line + sep)
        return True

    def write_json(self, file_path: str, data: dict) -> bool:
        """Write the data to the json file"""
        with open(file_path, 'w') as file:
            json.dump(data, file)
        return True

    def boolConversion(self, value: str, output=[True, False]) -> bool:
        """Convert the string to boolean
        
        input: 'true', 'yes', 'y', '1', 't' -> True
               'false', 'no', 'n', '0', 'f' -> False
        
        output: True or False
                1 or 0
                y or n
                t or f
        """
        if str(value).lower() in ['true', 'yes', 'y', '1', 't']:
            return output[0]
        elif str(value).lower() in ['false', 'no', 'n', '0', 'f']:
            return output[1]
        return None



import re
from Levenshtein import distance as levenshtein_distance
from sanatio.base_class import BaseValidator


class StringValidator(BaseValidator):

    def isAlphanumeric(self, value: str) -> bool:
        if value.isalnum():
            return True

    def isAlpha(self, value: str) -> bool:
        if value.isalpha():
            return True

    def isAscii(self, value) -> bool:
        if value.isascii():
            return True

    def isLength(self, value: str, min: int = 0, max: int = None) -> bool:
        """ check if the string length is between min and max """
        if min <= len(value) <= max:
            return True

    def isEmpty(self, value: str) -> bool:
        """ check if the string is empty or not """
        value = value.strip()
        if value == "":
            return True

    def contains(self, value: str, substring: str, ignoreCase: bool = False) -> bool:
        if ignoreCase:
            value = value.lower()
            substring = substring.lower()

        if substring in value:
            return True

    def levenshtein_distance(self, value1, value2):
        """ calculate distance between two strings """
        distance = levenshtein_distance(value1, value2)
        return distance

    def edit_distance(self, value1, value2):
        """ calculate distance between two strings """
        if len(value1) > len(value2):
            difference = len(value1) - len(value2)

        elif len(value2) > len(value1):
            difference = len(value2) - len(value1)

        else:
            difference = 0

        for val1, val2 in zip(value1, value2):
            if val1 != val2:
                difference += 1

        return difference

    def isEquals(self, value1: str, value2: str, ignoreCase: bool = False) -> bool:
        """ Check if the two string are equal or not """
        if self.isvalidString(value1) and self.isvalidString(value2) and \
                (ignoreCase and value1.lower() == value2.lower() or value1 == value2):
            return True

    def isVowel(self, value) -> bool:
        """ check if the string is Vowel or not """
        return value.lower() in ('a', 'e', 'i', 'o', 'u')

    def isConsonant(self, value) -> bool:
        """ check if the string is Consonant or not """
        return value.lower() not in ('a', 'e', 'i', 'o', 'u')

    def isPalindrome(self, value) -> bool:
        """ check if the string is Palindrome or not """
        return value == value[::-1]

    def trim(self, value):
        """ trim string """
        return value.strip() if self.isvalidString(value) else None

    def ltrim(self, value):
        return value.lstrip() if self.isvalidString(value) else None

    def rtrim(self, value):
        return value.rstrip() if self.isvalidString(value) else None

    def toUpperCase(self, value):
        """ convert string to upper case """
        return value.upper() if self.isvalidString(value) else None

    def toLowerCase(self, value):
        """ convert string to lower case """
        if self.isvalidString(value):
            return value.lower()

    def removeSymbols(self, value):
        """ remove symbols from string """
        return re.sub(r'[^\w\s]', '', value) if self.isvalidString(value) else None

    def removeNonASCII(self, value):
        """ remove non ASCII characters from string """
        if self.isvalidString(value):
            return value.encode("ascii", "ignore").decode()

    def removeNonWord(self, value):
        """ remove non word characters from string """
        return re.sub(r'[^\w]', '', value) if self.isvalidString(value) else None

    def removeNonNumeric(self, value):
        """ remove non numeric characters from string """
        return re.sub(r'[^\d]', '', value) if self.isvalidString(value) else None

    def removeTags(self, value):
        """ remove tags from string """
        return re.sub(r'<[^>]*>', '', value) if self.isvalidString(value) else None

    def removeWhiteSpace(self, value):
        """ remove white space from string """
        return value.replace(" ", "") if self.isvalidString(value) else None

    def removeProtocol(self, value):
        """ remove protocol from url """
        return re.sub(r'^https?:\/\/', '', value) if self.isvalidString(value) else None

    def removeProfanity(self, value):
        """ remove profanity from string """
        pass



import re


from sanatio.utils.utils import regexs_dict
from sanatio.base_class import BaseValidator


class UsernameValidator(BaseValidator):

    def isDiscordUsername(self, value: str) -> bool:
        regex = regexs_dict.get('discord_username_regex')
        if re.match(regex, value):
            return True
        return False



import re

from sanatio.base_class import BaseValidator


class PasswordValidator(BaseValidator):

    def isStrongPassword(self, value: str,
                         min_length: int = 8,
                         special_chars: bool = True,
                         numbers: bool = True,
                         uppercase: bool = True,
                         lowercase: bool = True) -> bool:
        """ check if the string is strong password or not

        requirements:
            1. At least 8 characters long
            2. At least one uppercase letter
            3. At least one lowercase letter
            4. At least one number
            5. At least one special character
        """
        if not self.isPasswordLength(value, min_length) or \
           (special_chars and not self.isPasswordSpecialChar(value)) or \
           (numbers and not self.isPasswordNumber(value)) or \
           (uppercase and not self.isPasswordUppercase(value)) or \
           (lowercase and not self.isPasswordLowercase(value)):
            return False

        return True

    def isPasswordUppercase(self, value: str, min_length: int = 1) -> bool:
        """ check if the string has uppercase letters """
        if len(re.findall(r'[A-Z]', value)) >= min_length:
            return True

    def isPasswordLowercase(self, value: str, min_length: int = 1) -> bool:
        """ check if the string has lowercase letters """
        if len(re.findall(r'[a-z]', value)) >= min_length:
            return True

    def isPasswordNumber(self, value: str, min_length: int = 1) -> bool:
        """ check if the string has numbers """
        if len(re.findall(r'[0-9]', value)) >= min_length:
            return True

    def isPasswordLength(self, value: str, min_length: int = 8) -> bool:
        """ check if the string length is between min and max """
        if len(value) >= min_length:
            return True

    def isPasswordSpecialChar(self, value: str, min_length: int = 1) -> bool:
        """ check if the string has special characters """
        if len(re.findall(r'[_@$]', value)) >= min_length:
            return True

    def isPasswordMatch(self, value: str, match_value: str, ignore_case: bool = False) -> bool:
        """ check if the string matches the match_value """
        if ignore_case and value.lower() == match_value.lower():
            return True
        elif value == match_value:
            return True

    def isPasswordNotMatch(self, value: str, match_value: str, ignore_case: bool = False) -> bool:
        """ check if the string does not match the match_value """
        if ignore_case and value.lower() != match_value.lower():
            return True
        elif value != match_value:
            return True

    def isPasswordNotInList(self, value: str, list_values: list, ignore_case: bool = False) -> bool:
        """ check if the string is not in the list """
        if ignore_case and value.lower() not in [x.lower() for x in list_values]:
            return True
        elif value not in list_values:
            return True

    def isPasswordNotInFile(self, value: str, file_path: str, ignore_case: bool = False) -> bool:
        """ check if the string is not in the file """
        data = self.read_file(file_path, split_lines=True)

        if ignore_case and value.lower() not in [x.lower() for x in data]:
            return True
        elif value not in data:
            return True



# Verhoeff's algorithm implementation for checksum digit calculation
verhoeff_table_d = (
    (0, 1, 2, 3, 4, 5, 6, 7, 8, 9),
    (1, 2, 3, 4, 0, 6, 7, 8, 9, 5),
    (2, 3, 4, 0, 1, 7, 8, 9, 5, 6),
    (3, 4, 0, 1, 2, 8, 9, 5, 6, 7),
    (4, 0, 1, 2, 3, 9, 5, 6, 7, 8),
    (5, 9, 8, 7, 6, 0, 4, 3, 2, 1),
    (6, 5, 9, 8, 7, 1, 0, 4, 3, 2),
    (7, 6, 5, 9, 8, 2, 1, 0, 4, 3),
    (8, 7, 6, 5, 9, 3, 2, 1, 0, 4),
    (9, 8, 7, 6, 5, 4, 3, 2, 1, 0))

verhoeff_table_p = (
    (0, 1, 2, 3, 4, 5, 6, 7, 8, 9),
    (1, 5, 7, 6, 2, 8, 3, 0, 9, 4),
    (5, 8, 0, 3, 7, 9, 6, 1, 4, 2),
    (8, 9, 1, 6, 0, 4, 3, 5, 2, 7),
    (9, 4, 5, 3, 1, 2, 6, 8, 7, 0),
    (4, 2, 8, 6, 5, 7, 3, 9, 0, 1),
    (2, 7, 9, 3, 8, 0, 6, 4, 1, 5),
    (7, 0, 4, 6, 9, 1, 3, 2, 5, 8))

verhoeff_table_inv = (0, 4, 3, 2, 1, 5, 6, 7, 8, 9)


class BaseChecksumAlgorithm:
    def __init__(self, input_value: str) -> None:
        self.input_value = input_value.replace(' ', '')

    def last_digit_and_remaining_numbers(self) -> tuple:
        """ Returns the last digit and the remaining numbers """
        return int(self.input_value[-1]), self.input_value[:-1]


class VerhoeffAlgorithm(BaseChecksumAlgorithm):

    def __checksum(self):
        """ For a given number returns a Verhoeff checksum digit """
        c = 0
        last_digit, remaining_numbers = self.last_digit_and_remaining_numbers()
        for i, item in enumerate(reversed(remaining_numbers)):
            c = verhoeff_table_d[c][verhoeff_table_p[(i + 1) % 8][int(item)]]

        if c == last_digit:
            return True

        return False

    def verify(self):
        """ Verify a given number including its checksum digit """
        return self.__checksum()


class EANCheckSum(BaseChecksumAlgorithm):

    def __checksum_ean13(self):
        """Calculate the checksum for the EAN code"""
        last_digit, remaining_numbers = self.last_digit_and_remaining_numbers()
        total_sum = sum(int(remaining_numbers[i]) * 3 if i % 2 != 0 else
                        int(remaining_numbers[i]) for i in range(len(remaining_numbers)))
        unit_digit = 0 if total_sum % 10 == 0 else 10 - (total_sum % 10)
        if unit_digit == last_digit:
            return True

    def __checksum_ean8(self) -> bool:
        """Calculate the checksum for the EAN code"""
        pass

    def verify(self) -> bool:
        """Check if the EAN code is valid"""
        return self.__checksum_ean13() if len(self.input_value) == 13 else self.__checksum_ean8()


class LuhnAlgorithm(BaseChecksumAlgorithm):
    """Class to validate a number using Luhn algorithm.

    Args:
        input_value (str): The input value to validate.

    returns:
        bool: True if the number is valid, False otherwise.
    """

    def __checksum(self) -> int:
        last_digit, remaining_numbers = self.last_digit_and_remaining_numbers()
        nums = [int(num) if idx % 2 != 0 else int(num) * 2 if int(num) * 2 <= 9
                else int(num) * 2 % 10 + int(num) * 2 // 10
                for idx, num in enumerate(reversed(remaining_numbers))]

        return (sum(nums) + last_digit) % 10 == 0

    def verify(self) -> bool:
        """Verify a number using Luhn algorithm."""
        return self.__checksum()


class Mod10Algorithm(LuhnAlgorithm):
    # luhn algorithm and mod10 algorithm are same
    pass



import os
import json

ROOT_DIR = os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
ASSETS_DIR = os.path.join(ROOT_DIR, 'sanatio/assets')


def load_json(filename):
    if not os.path.exists(filename):
        raise FileNotFoundError(f'File {filename} not found')

    if not filename.endswith('.json'):
        raise ValueError('Filename must be a json file')

    with open(os.path.join(filename), 'r', encoding="utf-8") as f:
        return json.load(f)


def load_asset(filename):
    return load_json(os.path.join(ASSETS_DIR, filename))


country_json = load_asset('country.json')
regexs_dict = load_asset('regex.json')



import re

from sanatio.utils.utils import country_json, regexs_dict
from sanatio.utils.checksum import VerhoeffAlgorithm
from sanatio.utils.checksum import LuhnAlgorithm
from sanatio.base_class import BaseValidator


class DocumentValidator(BaseValidator):
    """ 
    DocumentValidator class is used to validate document numbers like Aadhar card, License plate, Passport number, Credit card number

    Methods
    -------
    isAadharCard(value: str) -> bool:
        check if the string is Aadhar card or not

    isLicensePlate(value: str, locale: str) -> bool:
        check if the string is license plate or not

    isPassportNumber(value: str, locale: str) -> bool:
        check if the string is passport number or not

    isCreditCard(value: str) -> bool:
        check if the string is credit card number or not

    """

    def isAadharCard(self, value) -> bool:
        """ check if the string is Aadhar card or not """
        regex = regexs_dict.get('aadhar_regex')
        value = value.strip().replace(" ", "")
        if isinstance(value, int):
            value = str(value)
        if self.isLength(str(value), 12, 12) \
            and value[0] not in ['0', '1'] and re.match(regex, value) \
                and VerhoeffAlgorithm(value).verify():
            return True
        return False

    def isLicensePlate(self, value, locale: str) -> bool:
        """ check if the string is license plate or not """
        value = value.upper()
        country_data = country_json[locale]

        LicensePlate = country_data['LicensePlate']
        Format = LicensePlate['Format']
        Regex = LicensePlate['Regex']
        MinLength = LicensePlate['MinLength']
        MaxLength = LicensePlate['MaxLength']

        if re.match(Format, value) and re.match(Regex, value) \
                and self.isLength(value, MinLength, MaxLength):
            return True

        return False

    # TODO: research more about passport number
    def isPassportNumber(self, value, locale: str) -> bool:
        """ check if the string is passport number or not """
        country_data = country_json[locale]

        PassportNumberRegex = country_data['PassportNumberRegex']
        if re.match(PassportNumberRegex, value):
            return True
        return False

    def isCreditCard(self, value: str) -> bool:  # checksum not implemented
        regex = regexs_dict.get('credit_card_regex')
        if re.match(regex, value):
            if LuhnAlgorithm(value).verify():
                return True
        return False



import re

from sanatio.base_class import BaseValidator
from sanatio.utils.utils import regexs_dict


class NumberValidator(BaseValidator):

    def isDecimal(self, value: float) -> bool:
        """ check if the string is decimal or not """
        return self.isvalidNumber(value)

    def isDivisibleBy(self, number: int, divisor: int) -> bool:
        """ check if the number is divisible by divisor or not """
        return (self.isvalidNumber(number) and
                self.isvalidNumber(divisor) and
                divisor != 0 and
                number % divisor == 0)

    def truncate(self, value: float, digits: int) -> float:
        """ truncate the float value """
        regex = regexs_dict.get('truncate_regex')
        return float(re.findall(f'{regex}{{{digits}}}', str(value))[0])

    def toInt(self, value):
        """ convert string to int """
        try:
            return int(float(value))
        except ValueError:
            return None

    def toFloat(self, value):
        """ convert string to float """
        try:
            return float(value)
        except ValueError:
            return None

    def isPositive(self, value):
        """ check if the value is positive or not """
        return self.isvalidNumber(value) and value > 0

    def isNegative(self, value):
        """ check if the value is negative or not """
        return self.isvalidNumber(value) and value < 0

    def isInteger(self, value):
        """ check if the value is integer or not """
        return self.isvalidNumber(value) and isinstance(value, int)

    def isFloat(self, value):
        """ check if the value is float or not """
        return self.isvalidNumber(value) and isinstance(value, float)

    def isZero(self, value):
        """ check if the value is zero or not """
        return self.isvalidNumber(value) and value == 0

    def isNonZero(self, value):
        """ check if the value is non-zero or not """
        return self.isvalidNumber(value) and value != 0

    def isGreaterThan(self, value, other):
        """ check if the value is greater than other or not """
        return (self.isvalidNumber(value) and
                self.isvalidNumber(other) and
                value > other)

    def isGreaterThanOrEqual(self, value, other):
        """ check if the value is greater than or equal to other or not """
        return (self.isvalidNumber(value) and
                self.isvalidNumber(other) and
                value >= other)

    def isLessThan(self, value, other):
        """ check if the value is less than other or not """
        return (self.isvalidNumber(value) and
                self.isvalidNumber(other) and
                value < other)

    def isLessThanOrEqual(self, value, other):
        """ check if the value is less than or equal to other or not """
        return (self.isvalidNumber(value) and
                self.isvalidNumber(other) and
                value <= other)

    def isBetween(self, value, min_value, max_value):
        """ check if the value is between min_value and max_value or not """
        return (self.isvalidNumber(value) and
                self.isvalidNumber(min_value) and
                self.isvalidNumber(max_value) and
                min_value <= value <= max_value)

    def isPrime(self, value) -> bool:
        """ check if the value is a prime number or not """
        if not self.isvalidNumber(value) or value < 2:
            return False
        for i in range(2, int(value ** 0.5) + 1):
            if value % i == 0:
                return False
        return True

    def isEven(self, value) -> bool:
        """ check if the value is even or not """
        return self.isvalidNumber(value) and value % 2 == 0

    def isOdd(self, value) -> bool:
        """ check if the value is odd or not """
        return self.isvalidNumber(value) and value % 2 != 0

    def isMultipleOf(self, value, multiple) -> bool:
        """ check if the value is a multiple of another number """
        if (not self.isvalidNumber(value) or
                not self.isvalidNumber(multiple) or
                multiple == 0):
            return False
        return value % multiple == 0

    def isSquare(self, value) -> bool:
        """ Check if the value is a perfect square or not """
        if not self.isvalidNumber(value):
            return False
        if value < 0:
            return False
        square_root = round(value ** 0.5)
        return square_root ** 2 == value

    def isCube(self, value) -> bool:
        """ Check if the value is a perfect cube or not """
        if not self.isvalidNumber(value):
            return False
        if value < 0:
            cube_root = round((-value) ** (1/3))
            return -cube_root ** 3 == value
        else:
            cube_root = round(value ** (1/3))
            return cube_root ** 3 == value



from datetime import datetime
from dateutil.parser import parse
from dateutil import relativedelta

from sanatio.base_class import BaseValidator


class DateValidator(BaseValidator):

    def isLeapYear(self, year) -> bool:
        """ check if the year is leap year or not """
        if isinstance(year, str):
            year = int(year)
        return year % 4 == 0 and (year % 100 != 0 or year % 400 == 0)

    def isAfter(self, date1, date2=datetime.now(), format='%Y-%m-%d') -> bool:
        """ check if date1 is after date2 default date2 is current date

        Args:
            date1 (str): date in string format
            date2 (str): date in string format default is current date

        """
        if isinstance(date1, str):
            date1 = datetime.strptime(date1, format)
        if isinstance(date2, str):
            date2 = datetime.strptime(date2, format)

        return date1 > date2

    def isBefore(self, date1, date2=datetime.now(), format='%Y-%m-%d') -> bool:
        """
        check if date1 is before date2 default date2 is current date

        Args:
            date1 (str): date in string format
            date2 (str): date in string format default is current date
        """
        if isinstance(date1, str):
            date1 = datetime.strptime(date1, format)
        if isinstance(date2, str):
            date2 = datetime.strptime(date2, format)

        return date1 < date2

    def isDate(self, value) -> bool:
        """ check if the string is date or not """
        if not self.isvalidString(value):
            return False
        try:
            date_obj = parse(value)
            if date_obj:
                return True
        except Exception:
            return False
        return False

    def toDate(self, value, format='%Y-%m-%d'):  # need to improve this function
        """ convert string to date """
        return datetime.strptime(value, format) if self.isDate(value) else None

    def getYearDiff(self, date1, date2, increment=False, date_format="%d-%m-%Y"):
        """ get the difference between two dates in years """
        if isinstance(date1, str):
            date1 = datetime.strptime(date1, date_format)

        if isinstance(date2, str):
            date2 = datetime.strptime(date2, date_format)

        diff = relativedelta.relativedelta(date1, date2)
        total_days = abs(diff.years) * 365 + abs(diff.months) * 30 + abs(diff.days) if diff else 0
        total_years = total_days // 364
        if total_days % 364 > 0 and increment:
            total_years += 1

        return total_years



from sanatio.base_class import BaseValidator


class ArrayValidator(BaseValidator):

    def isArray(self, value) -> bool:
        """ check if the string is array or not """
        return isinstance(value, list)

    def isArrayLength(self, value, min: int = 0, max: int = None) -> bool:
        """ check if the array length is between min and max """
        return min <= len(value) <= max

    def isContains(self, value, contains) -> bool:
        """ check if the array contains the element or not """
        return contains in value

    def isUnique(self, value) -> bool:
        """ check if the array contains unique elements or not """
        return len(value) == len(set(value))

    def isMultidimensional(self, value) -> bool:
        """ check if the array is multidimensional or not """
        return any(isinstance(i, list) for i in value)

    def isMinlength(self, value, min: int) -> bool:
        """ check if the array length is greater than min """
        return len(value) > min

    def isMaxlength(self, value, max: int) -> bool:
        """ check if the array length is less than max """
        return len(value) < max
